%
%   ESTE PROGRAMA ESTIMA EL MODELO EXPONENCIAL, SIN TRMINO CONSTANTE
%
%
clear;

%%%%%%%%%%%%%%%%%%%% simulando la ecuacin %%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%% y(t)=alfa*exp(beta*x(t))+u(t)%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
T=200;                                    % nmero de datos
xm = 3;                                   % media de X
sigmapi=2;                                % desviacin tpica de la innovacin
x = xm+normrnd(0,sigmapi,T,1);            % generacin de la variable x
shock = normrnd(0,1,T+1000,1);            % generacin de una normal(0,1)
shock = 3*shock(1001:1000+T,1);           % perturbacin aleatoria (u_t)
alfat = 3.4;                              % parmetro alfa 
betat = -.5;                              % parmetro beta 
y = alfat.*exp(betat.*x)+shock;           % generacin de la variable Y 
display('Valores verdaderos:  alpha  beta')
[alfat  betat]
global vari                               % declaramos los datos simulados como variables globales
vari=[y x];
plot(x,y,'b.');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
RES = [];
for i = -10:1:10
beta0=i;
aux = exp(beta0*x);
alfa0 = mean(y.*aux)/mean(aux.^2);
res = y - alfa0.*aux;
suma = res'*res;
RES = [RES; suma  alfa0  beta0];
end
plot(RES(:,1))
[f,c] = find(RES==min(RES(:,1)));
display('Estimaciones a partir de las CPO evaluadas en una rejilla de valores de alfa')
RES(f,2:3)
pause
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
beta0 = RES(f,3);
alpha0 = mean(y.*exp(beta0*x))/mean(exp(2*beta0*x));
theta0 = [alpha0; beta0];
% Minimizamos la Suma de Cuadrados de Residuos directamente. Esta SCR se
% calcula en el archivo 'srdemd.m'
options=optimset('Display','Iter','MaxFunEvals',100,'MaxIter',100,'TolFun',0.00001);
theta1=fminunc('srdemd',theta0,options);  
alfaest1=theta1(1);                % estimacin de alfa
betaest1=theta1(2);                % estimacin de beta
display('Estimaciones de alfa y beta minimizando la Suma de Cuadrados de Residuos')
[alfaest1   betaest1]
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculamos numricamente los parmetros resolviendo las
% condiciones de primer orden del problema de minimizacin
% con la siguiente sentencia encontramos un 'cero' para el sistema de ecuaciones de
% primer orden del problema de minimizacin de la suma residual. Las ecuaciones que forman
% tal sistema se encuentran en el archivo 'cpo.m'
theta1=fsolve('cpo_demdir',theta0); 
alfaest2=theta1(1);          % estimacin de alfa
betaest2=theta1(2);          % estimacin de beta
display('Estimaciones de alfa y beta minimizando resolviendo las condiciones de primer orden')
[alfaest2   betaest2]
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Calculamos numricamente los parmetros resolviendo por
%%%%%%%%% Newton-Raphson, con derivadas analticas.
itermax=100;       % mximo nmero de iteraciones
epsilon=1e-6;      % error admisible del criterio de parada
Gtheta=[];         % matriz donde acumulamos los valores de alfa y beta en cada iteracin
longitud = 1;
theta_obj = theta0;
for iter=1:itermax
    Grad=feval('cpo_demdir',theta_obj);  % calculamos el gradiente usando las derivadas analticas 
        % que se encuentran en el archivo 'cpo.m'                      
    hesst=feval('cso_demdir',theta_obj); % calculamos el hessiano usando las derivadas segundas 
        % analticas que se encuentran en el archivo 'cso.m
    Hesst=[hesst(1) hesst(2); hesst(3) hesst(4)];  % formamos matriz hessiana
    theta1=theta_obj-longitud.*inv(Hesst)*Grad; % aplicamos el algoritmo de Gauss-Newton
    Gtheta=[Gtheta;theta1'];   % almacenamos los valores de alfa y beta de cada iteracin                                 
    % criterio de parada
    if norm(theta1-theta_obj)<epsilon
        iter
        break
    else
        theta_obj=theta1;
    end
end
alfaest3=theta1(1);           % estimacin de alfa
betaest3=theta1(2);           % estimacin de beta
sest=feval('srdemd',theta1)/(T-2); % Varianza del trmino de error
MCOV=sest*inv(Hesst);              % estimacin de la matriz de covarianzas de theta
MDT=diag(MCOV.^0.5);
display('Estimaciones de alfa y beta por Newton-Raphson con derivadas analiticas')
display('Primera fila: estimaciones; Segunda fila: DT')
[alfaest3   betaest3;  MDT(1)   MDT(2)]
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%%%%%%% Calculamos numricamente los parmetros resolviendo por
% %%%%%%%%% Newton-Raphson, con derivadas analticas.
% itermax=100;       % mximo nmero de iteraciones
% epsilon=1e-6;      % error admisible del criterio de parada
% Gtheta=[];         % matriz donde acumulamos los valores de alfa y beta en cada iteracin
% longitud = 1;
% theta_obj = theta0;
% for iter=1:itermax
%     Grad=feval('cpo_demdir',theta_obj);  % calculamos el gradiente usando las derivadas analticas 
%         % que se encuentran en el archivo 'cpo.m'                      
%     theta1=theta_obj-longitud.*inv(Grad*Grad')*Grad; % aplicamos el algoritmo de Gauss-Newton
%     Gtheta=[Gtheta;theta1'];   % almacenamos los valores de alfa y beta de cada iteracin                                 
%     % criterio de parada
%     if norm(theta1-theta_obj)<epsilon
%         iter
%         break
%     else
%         theta_obj=theta1;
%     end
% end
% alfaest3bis=theta1(1);           % estimacin de alfa
% betaest3bis=theta1(2);           % estimacin de beta
% sest=feval('srdemd',theta1)/(T-2); % Varianza del trmino de error
% MCOV=sest*inv(Grad*Grad');              % estimacin de la matriz de covarianzas de theta
% MDT=diag(MCOV.^0.5);
% display('Estimaciones de alfa y beta por Newton-Raphson con derivadas analiticas')
% display('Primera fila: estimaciones; Segunda fila: DT')
% [alfaest3bis   betaest3bis;  MDT(1)   MDT(2)]
% pause;
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Calculamos numricamente los parmetros resolviendo por 
%%%%%%%%% por Newton-Raphson, con derivadas numricas.
itermax=100;  % mximo nmero de iteraciones
Gtheta2=[];    % matriz donde acumulamos los valores de alfa y beta en cada iteracin
longitud = .10;
theta_obj = theta0;
for iter2=1:itermax
    auxi=1e-8*eye(2);
    theta01=theta_obj+auxi(:,1);
    theta02=theta_obj+auxi(:,2);    
    %calculando el gradiente numricamente
    Gradn=zeros(2,1);
    for i=1:2
        Gradn(i)=(feval('srdemd',theta_obj+auxi(:,i))-feval('srdemd',theta_obj))/auxi(i,i);
    end
    %calculando el hessiano nmericamente
    Hessn=zeros(2,2);
    auxi=1e-5*eye(2);
    for i=1:2
        for j=1:2
            Hessn(i,j)=(feval('srdemd',theta_obj+auxi(:,i)+auxi(:,j))-...
                feval('srdemd',theta_obj+auxi(:,i))-...
                feval('srdemd',theta_obj+auxi(:,j))+...
                feval('srdemd',theta_obj))/(auxi(j,j)^2);
        end
    end
    % calculando el nuevo theta0
    theta1=theta_obj-longitud.*inv(Hessn)*Gradn;  %aplicamos el algoritmo Gauss-Newton  
    Gtheta2=[Gtheta2;theta1'];         % almacenamos las estimaciones de alfa
                                       % y beta de cada iteracin
    % criterio de parada
    if norm(theta1-theta_obj)<epsilon
            iter2
        break
    else
        theta_obj=theta1;
    end
end 
alfaest4=theta1(1);          % estimacin de alfa
betaest4=theta1(2);          % estimacin de beta
sest2=feval('srdemd',theta1)/(T-2); % Varianza del trmino de error
MCOV2=sest2*inv(Hessn);              % estimacin de la matriz de covarianzas de theta
MDT2=diag(MCOV2.^0.5);
display('Estimaciones de alfa y beta por Newton-Raphson con derivadas numricas')
[alfaest4   betaest4;  MDT2(1)    MDT2(2)]
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simplex method
% theta0=[10; -0.5];
theta=simpxjra('srdemd',theta0');
alfaest5=theta(1);
betaest5=theta(2);
disp('usando algoritmo simplex_jra')
[alfaest5  betaest5]
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% theta0=[10; -0.5];
theta=fminsearch('srdemd',theta0',options);
alfaest6=theta(1);
betaest6=theta(2);
disp('usando algoritmo simplex_Matlab')
[alfaest6  betaest6]
pause;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[alfaest1  betaest1; alfaest2  betaest2;  alfaest3  betaest3; alfaest4  betaest4; alfaest5  betaest5;  alfaest6   betaest6]


